
/*
Basic Quo Module

@namespace Quo
@class Ajax

@author Javier Jimenez Villar <javi@tapquo.com> || @soyjavi
 */

(function() {
  "use strict";
  (function($$) {
    var DEFAULT, JSONP_ID, MIME_TYPES, _isJsonP, _jsonp, _xhrError, _xhrForm, _xhrHeaders, _xhrStatus, _xhrSuccess, _xhrTimeout;
    DEFAULT = {
      TYPE: "GET",
      MIME: "json"
    };
    MIME_TYPES = {
      script: "text/javascript, application/javascript",
      json: "application/json",
      xml: "application/xml, text/xml",
      html: "text/html",
      text: "text/plain"
    };
    JSONP_ID = 0;
    $$.ajaxSettings = {
      type: DEFAULT.TYPE,
      async: true,
      success: {},
      error: {},
      context: null,
      dataType: DEFAULT.MIME,
      headers: {},
      xhr: function() {
        return new window.XMLHttpRequest();
      },
      crossDomain: false,
      timeout: 0
    };

    /*
    Perform an asynchronous HTTP (Ajax) request.
    @method ajax
    @param  {object} A set of key/value pairs that configure the Ajax request
     */
    $$.ajax = function(options) {
      var abortTimeout, error, settings, xhr;
      settings = $$.mix($$.ajaxSettings, options);
      if (settings.type === DEFAULT.TYPE) {
        settings.url += $$.serialize(settings.data, "?");
      } else {
        settings.data = $$.serialize(settings.data);
      }
      if (_isJsonP(settings.url)) {
        return _jsonp(settings);
      }
      xhr = settings.xhr();
      xhr.onreadystatechange = function() {
        if (xhr.readyState === 4) {
          clearTimeout(abortTimeout);
          return _xhrStatus(xhr, settings);
        }
      };
      xhr.open(settings.type, settings.url, settings.async);
      _xhrHeaders(xhr, settings);
      if (settings.timeout > 0) {
        abortTimeout = setTimeout((function() {
          return _xhrTimeout(xhr, settings);
        }), settings.timeout);
      }
      try {
        xhr.send(settings.data);
      } catch (_error) {
        error = _error;
        xhr = error;
        _xhrError("Resource not found", xhr, settings);
      }
      return xhr;
    };

    /*
    Load data from the server using a HTTP GET request.
    @method get
    @param  {string} A string containing the URL to which the request is sent.
    @param  {string} [OPTIONAL] A plain object or string that is sent to the server with the request.
    @param  {string} [OPTIONAL] A callback function that is executed if the request succeeds.
    @param  {string} [OPTIONAL] The type of data expected from the server
     */
    $$.get = function(url, data, success, dataType) {
      return $$.ajax({
        url: url,
        data: data,
        success: success,
        dataType: dataType
      });
    };

    /*
    Load data from the server using a HTTP POST request.
    @method post
    @param  {string} A string containing the URL to which the request is sent.
    @param  {string} [OPTIONAL] A plain object or string that is sent to the server with the request.
    @param  {string} [OPTIONAL] A callback function that is executed if the request succeeds.
    @param  {string} [OPTIONAL] The type of data expected from the server
     */
    $$.post = function(url, data, success, dataType) {
      return _xhrForm("POST", url, data, success, dataType);
    };

    /*
    Load data from the server using a HTTP PPUTOST request.
    @method put
    @param  {string} A string containing the URL to which the request is sent.
    @param  {string} [OPTIONAL] A plain object or string that is sent to the server with the request.
    @param  {string} [OPTIONAL] A callback function that is executed if the request succeeds.
    @param  {string} [OPTIONAL] The type of data expected from the server
     */
    $$.put = function(url, data, success, dataType) {
      return _xhrForm("PUT", url, data, success, dataType);
    };

    /*
    Load data from the server using a HTTP DELETE request.
    @method delete
    @param  {string} A string containing the URL to which the request is sent.
    @param  {string} [OPTIONAL] A plain object or string that is sent to the server with the request.
    @param  {string} [OPTIONAL] A callback function that is executed if the request succeeds.
    @param  {string} [OPTIONAL] The type of data expected from the server
     */
    $$["delete"] = function(url, data, success, dataType) {
      return _xhrForm("DELETE", url, data, success, dataType);
    };

    /*
    Load JSON-encoded data from the server using a GET HTTP request.
    @method json
    @param  {string} A string containing the URL to which the request is sent.
    @param  {string} [OPTIONAL] A plain object or string that is sent to the server with the request.
    @param  {string} [OPTIONAL] A callback function that is executed if the request succeeds.
     */
    $$.json = function(url, data, success) {
      return $$.ajax({
        url: url,
        data: data,
        success: success
      });
    };

    /*
    Encode a set of form elements as a string for submission.
    @method serialize
    @param  {object}
     */
    $$.serialize = function(parameters, character) {
      var parameter, serialize;
      if (character == null) {
        character = "";
      }
      serialize = character;
      for (parameter in parameters) {
        if (parameters.hasOwnProperty(parameter)) {
          if (serialize !== character) {
            serialize += "&";
          }
          serialize += "" + (encodeURIComponent(parameter)) + "=" + (encodeURIComponent(parameters[parameter]));
        }
      }
      if (serialize === character) {
        return "";
      } else {
        return serialize;
      }
    };
    _jsonp = function(settings) {
      var abortTimeout, callbackName, script, xhr;
      if (settings.async) {
        callbackName = "jsonp" + (++JSONP_ID);
        script = document.createElement("script");
        xhr = {
          abort: function() {
            $$(script).remove();
            if (callbackName in window) {
              return window[callbackName] = {};
            }
          }
        };
        abortTimeout = void 0;
        window[callbackName] = function(response) {
          clearTimeout(abortTimeout);
          $$(script).remove();
          delete window[callbackName];
          return _xhrSuccess(response, settings);
        };
        script.src = settings.url.replace(RegExp("=\\?"), "=" + callbackName);
        $$("head").append(script);
        if (settings.timeout > 0) {
          abortTimeout = setTimeout((function() {
            return _xhrTimeout(xhr, settings);
          }), settings.timeout);
        }
        return xhr;
      } else {
        return console.error("QuoJS.ajax: Unable to make jsonp synchronous call.");
      }
    };
    _xhrStatus = function(xhr, settings) {
      if ((xhr.status >= 200 && xhr.status < 300) || xhr.status === 0) {
        if (settings.async) {
          _xhrSuccess(xhr, settings);
        }
      } else {
        _xhrError("QuoJS.ajax: Unsuccesful request", xhr, settings);
      }
    };
    _xhrSuccess = function(xhr, settings) {
      settings.success.call(settings.context, xhr);
    };
    _xhrError = function(type, xhr, settings) {
      settings.error.call(settings.context, type, xhr, settings);
    };
    _xhrHeaders = function(xhr, settings) {
      var header;
      if (settings.contentType) {
        settings.headers["Content-Type"] = settings.contentType;
      }
      if (settings.dataType) {
        settings.headers["Accept"] = MIME_TYPES[settings.dataType];
      }
      for (header in settings.headers) {
        xhr.setRequestHeader(header, settings.headers[header]);
      }
    };
    _xhrTimeout = function(xhr, settings) {
      xhr.onreadystatechange = {};
      xhr.abort();
      _xhrError("QuoJS.ajax: Timeout exceeded", xhr, settings);
    };
    _xhrForm = function(method, url, data, success, dataType) {
      return $$.ajax({
        type: method,
        url: url,
        data: data,
        success: success,
        dataType: dataType,
        contentType: "application/x-www-form-urlencoded"
      });
    };
    return _isJsonP = function(url) {
      return RegExp("=\\?").test(url);
    };
  })(Quo);

}).call(this);
