
/*
Basic Quo Module

@namespace Quo
@class css

@author Javier Jimenez Villar <javi@tapquo.com> || @soyjavi
 */

(function() {
  "use strict";
  (function($$) {
    var VENDORS, _computedStyle;
    VENDORS = ["-webkit-", "-moz-", "-ms-", "-o-", ""];

    /*
    Add class to a given instance element
    @method addClass
    @param  {string} Name of stylesheet class
     */
    $$.fn.addClass = function(name) {
      return this.each(function() {
        return this.classList.add(name);
      });
    };

    /*
    Remove stylesheet class to a given instance element
    @method addClass
    @param  {string} Name of stylesheet class
     */
    $$.fn.removeClass = function(name) {
      return this.each(function() {
        return this.classList.remove(name);
      });
    };

    /*
    Toggle stylesheet class to a given instance element
    @method addClass
    @param  {string} Name of stylesheet class
     */
    $$.fn.toggleClass = function(name) {
      return this.each(function() {
        var method;
        method = this.classList.contains(name) ? "remove" : "add";
        return this.classList[method](name);
      });
    };

    /*
    Test if a stylesheet class is in the giben instance element
    @method hasClass
    @param  {string} Name of stylesheet class
     */
    $$.fn.hasClass = function(name) {
      return this.length > 0 && this[0].classList.contains(name);
    };

    /*
    List a object with all classes in a given instance element
    @method listClass
    @param  {string} Name of stylesheet class
     */
    $$.fn.listClass = function() {
      if (this.length > 0) {
        return this[0].classList;
      }
    };

    /*
    Set/Get a stylesheet property in a given instance element
    @method style
    @param  {string} Name of property
    @param  {string} [OPTIONAL] Value for property
     */
    $$.fn.style = $$.fn.css = function(property, value) {
      var el;
      if (value != null) {
        return this.each(function() {
          return this.style[property] = value;
        });
      } else {
        el = this[0];
        return el.style[property] || _computedStyle(el, property);
      }
    };

    /*
    Set/Get a stylesheet vendor-prefix property in a given instance element
    @method vendor
    @param  {string} Name of property
    @param  {string} Value for property
     */
    $$.fn.vendor = function(property, value) {
      var prefix, _i, _len, _results;
      _results = [];
      for (_i = 0, _len = VENDORS.length; _i < _len; _i++) {
        prefix = VENDORS[_i];
        _results.push(this.style("" + prefix + property, value));
      }
      return _results;
    };
    return _computedStyle = function(element, property) {
      return document.defaultView.getComputedStyle(element, "")[property];
    };
  })(Quo);

}).call(this);
