
/*
Basic Quo Module

@namespace Quo
@class Element

@author Javier Jimenez Villar <javi@tapquo.com> || @soyjavi
 */

(function() {
  "use strict";
  (function($$) {

    /*
    Get/Set attribute to a given instance element
    @method attr
    @param  {string} Name of attribute
    @param  {string} [OPTIONAL] Value of attribute
     */
    $$.fn.attr = function(name, value) {
      if (this.length > 0 && $$.toType(name) === "string") {
        if (value) {
          return this.each(function() {
            return this.setAttribute(name, value);
          });
        } else {
          return this[0].getAttribute(name);
        }
      }
    };

    /*
    Remove attribute to a given instance element
    @method removeAttr
    @param  {string} Name of attribute
     */
    $$.fn.removeAttr = function(name) {
      if (this.length > 0 && $$.toType(name) === "string") {
        return this.each(function() {
          return this.removeAttribute(name);
        });
      }
    };

    /*
    Get/Set data attribute to a given instance element
    @method data
    @param  {string} Name of data attribute
    @param  {string} [OPTIONAL] Value of data atribbute
     */
    $$.fn.data = function(name, value) {
      return this.attr("data-" + name, value);
    };

    /*
    Remove data attribute to a given instance element
    @method removeAttr
    @param  {string} Name of data attribute
     */
    $$.fn.removeData = function(name) {
      return this.removeAttr("data-" + name);
    };

    /*
    Remove data attribute to a given instance element
    @method val
    @param  {string} Name of data attribute
     */
    $$.fn.val = function(value) {
      if (value != null) {
        return this.each(function() {
          return this.value = value.toString();
        });
      } else {
        if (this.length > 0) {
          return this[0].value;
        } else {
          return null;
        }
      }
    };

    /*
    Shows a given instance element
    @method show
     */
    $$.fn.show = function() {
      return this.style("display", "block");
    };

    /*
    Hides a given instance element
    @method hide
     */
    $$.fn.hide = function() {
      return this.style("display", "none");
    };

    /*
    Trigger that event on an element
    @method focus
     */
    $$.fn.focus = function() {
      return this[0].focus();
    };

    /*
    Trigger that event on an element
    @method blur
     */
    $$.fn.blur = function() {
      return this[0].blur();
    };

    /*
    Get a offset of a given instance element
    @method offset
     */
    return $$.fn.offset = function() {
      var bounding, offset;
      if (this.length > 0) {
        bounding = this[0].getBoundingClientRect();
        offset = {
          left: bounding.left + window.pageXOffset,
          top: bounding.top + window.pageYOffset,
          width: bounding.width,
          height: bounding.height
        };
      }
      return offset;
    };
  })(Quo);

}).call(this);
