
/*
Basic Quo Module

@namespace Quo
@class Environment

@author Javier Jimenez Villar <javi@tapquo.com> || @soyjavi
 */

(function() {
  "use strict";
  (function($$) {
    var IS_WEBKIT, SUPPORTED_OS, _current, _detectBrowser, _detectOS, _detectScreen;
    _current = null;
    IS_WEBKIT = /WebKit\/([\d.]+)/;
    SUPPORTED_OS = {
      Android: /(Android)\s+([\d.]+)/,
      ipad: /(iPad).*OS\s([\d_]+)/,
      iphone: /(iPhone\sOS)\s([\d_]+)/,
      Blackberry: /(BlackBerry|BB10|Playbook).*Version\/([\d.]+)/,
      FirefoxOS: /(Mozilla).*Mobile[^\/]*\/([\d\.]*)/,
      webOS: /(webOS|hpwOS)[\s\/]([\d.]+)/
    };

    /*
    Remove attribute to a given instance element
    @method isMobile
    @return {boolean} True if it's mobile, False if not.
     */
    $$.isMobile = function() {
      this.environment();
      return _current.isMobile;
    };

    /*
    Remove attribute to a given instance element
    @method environment
    @return {object} Environment attributes
     */
    $$.environment = function() {
      var os, user_agent;
      if (!_current) {
        user_agent = navigator.userAgent;
        os = _detectOS(user_agent);
        _current = {
          browser: _detectBrowser(user_agent),
          isMobile: !!os,
          screen: _detectScreen(),
          os: os
        };
      }
      return _current;
    };
    _detectBrowser = function(user_agent) {
      var webkit;
      webkit = user_agent.match(IS_WEBKIT);
      if (webkit) {
        return webkit[0];
      } else {
        return user_agent;
      }
    };
    _detectOS = function(user_agent) {
      var detected_os, os, supported;
      for (os in SUPPORTED_OS) {
        supported = user_agent.match(SUPPORTED_OS[os]);
        if (supported) {
          detected_os = {
            name: os === "iphone" || os === "ipad" || os === "ipod" ? "ios" : os,
            version: supported[2].replace("_", ".")
          };
          break;
        }
      }
      return detected_os;
    };
    return _detectScreen = function() {
      return {
        width: window.innerWidth,
        height: window.innerHeight
      };
    };
  })(Quo);

}).call(this);
