
/*
Quo Gestures manager

@namespace Quo
@class Gestures

@author Javier Jimenez Villar <javi@tapquo.com> || @soyjavi
 */

(function() {
  "use strict";
  var __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  Quo.Gestures = (function($$) {
    var add, trigger, _addDelegations, _cancel, _disabled_tags, _end, _fingers, _getFingers, _handle, _handlers, _move, _originalEvent, _start, _started;
    _started = false;
    _handlers = {};
    _fingers = null;
    _originalEvent = null;
    _disabled_tags = ["input", "select", "textarea"];
    $$(document).ready(function() {
      var environment;
      environment = $$(document.body);
      environment.bind("touchstart", _start);
      environment.bind("touchmove", _move);
      environment.bind("touchend", _end);
      return environment.bind("touchcancel", _cancel);
    });
    add = function(gesture) {
      _handlers[gesture.name] = gesture.handler;
      return _addDelegations(gesture.events);
    };
    trigger = function(target, eventName, gestureData) {
      return $$(target).trigger(eventName, gestureData, _originalEvent);
    };
    _start = function(ev) {
      var _ref;
      if (_ref = ev.srcElement.tagName.toLowerCase(), __indexOf.call(_disabled_tags, _ref) >= 0) {
        return ev.stopPropagation();
      }
      _started = true;
      _originalEvent = ev || event;
      _fingers = _getFingers(ev);
      return _handle("start", ev.target, _fingers);
    };
    _move = function(ev) {
      if (!_started) {
        return;
      }
      _originalEvent = ev || event;
      _fingers = _getFingers(ev);
      if (_fingers.length > 1) {
        _originalEvent.preventDefault();
      }
      return _handle("move", ev.target, _fingers);
    };
    _end = function(ev) {
      if (!_started) {
        return;
      }
      _originalEvent = ev || event;
      _handle("end", ev.target, _fingers);
      return _started = false;
    };
    _cancel = function(ev) {
      _started = false;
      return _handle("cancel");
    };
    _addDelegations = function(events) {
      events.forEach(function(event_name) {
        return $$.fn[event_name] = function(callback) {
          return $$(document.body).delegate(this.selector, event_name, callback);
        };
      });
      return this;
    };
    _handle = function(event, target, data) {
      var handler, name, _results;
      _results = [];
      for (name in _handlers) {
        handler = _handlers[name];
        if (handler[event]) {
          _results.push(handler[event].call(handler, target, data));
        }
      }
      return _results;
    };
    _getFingers = function(event) {
      var t, _i, _len, _ref, _results;
      _ref = event.touches || [event];
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        t = _ref[_i];
        _results.push({
          x: t.pageX,
          y: t.pageY
        });
      }
      return _results;
    };
    return {
      add: add,
      trigger: trigger
    };
  })(Quo);

}).call(this);


/*
Quo Basic Gestures: tap, hold, singleTap, doubleTap

@namespace Quo.Gestures
@class Basic

@author Ignacio Olalde Ramos <ina@tapquo.com> || @piniphone
 */

(function() {
  "use strict";
  Quo.Gestures.add({
    name: "basic",
    events: ["tap", "hold", "singleTap", "doubleTap", "touch"],
    handler: (function(base) {
      var DELAY, GAP, cancel, end, move, start, _hold_timeout, _last_tap, _simpletap_timeout, _start, _target, _valid;
      GAP = 15;
      DELAY = {
        TAP: 250,
        DOUBLE_TAP: 400,
        HOLD: 400
      };
      _hold_timeout = null;
      _simpletap_timeout = null;
      _valid = true;
      _target = null;
      _start = null;
      _last_tap = null;
      start = function(target, data) {
        if (data.length === 1) {
          _start = {
            time: new Date(),
            x: data[0].x,
            y: data[0].y
          };
          _target = target;
          return _hold_timeout = setTimeout(function() {
            return base.trigger(target, "hold", data[0]);
          }, DELAY.HOLD);
        } else {
          return cancel();
        }
      };
      move = function(target, data) {
        var xDiff, yDiff;
        if (_start !== null) {
          xDiff = data[0].x - _start.x;
          yDiff = data[0].y - _start.y;
          if (xDiff > GAP || yDiff > GAP || data.length > 1) {
            return cancel();
          }
        }
      };
      end = function(target, data) {
        var now;
        base.trigger(target, "touch", data[0]);
        if (!_start) {
          return;
        }
        clearTimeout(_hold_timeout);
        now = new Date();
        if ((now - _start.time) < DELAY.TAP) {
          if ((now - _last_tap) < DELAY.DOUBLE_TAP) {
            clearTimeout(_simpletap_timeout);
            base.trigger(target, "doubleTap", data[0]);
            return _last_tap = null;
          } else {
            _last_tap = now;
            base.trigger(target, "tap", data[0]);
            return _simpletap_timeout = setTimeout(function() {
              return base.trigger(target, "singleTap", data[0]);
            }, DELAY.DOUBLE_TAP + 5);
          }
        }
      };
      cancel = function() {
        _start = null;
        _valid = false;
        clearTimeout(_hold_timeout);
        return clearTimeout(_simpletap_timeout);
      };
      return {
        start: start,
        move: move,
        end: end,
        cancel: cancel
      };
    })(Quo.Gestures)
  });

}).call(this);


/*
Quo Drag Gestures: drag, dragging

@namespace Quo.Gestures
@class Drag

@author Ignacio Olalde Ramos <ina@tapquo.com> || @piniphone
 */

(function() {
  "use strict";
  Quo.Gestures.add({
    name: "drag",
    events: ["drag", "dragging"],
    handler: (function(base) {
      var GAP, end, move, start, _average, _calcDelta, _check, _last, _num_fingers, _start, _target;
      GAP = 20;
      _target = null;
      _num_fingers = null;
      _start = null;
      _last = null;
      start = function(target, data) {
        if (data.length >= 2) {
          _target = target;
          _num_fingers = data.length;
          return _start = _average(data);
        }
      };
      move = function(target, data) {
        var delta;
        if (data.length === _num_fingers) {
          delta = _calcDelta(data);
          _last = {
            touches: data,
            delta: delta
          };
          return _check(true);
        }
      };
      end = function(target, data) {
        if (_start && _last) {
          _check(false);
          _num_fingers = null;
          _start = null;
          return _last = null;
        }
      };
      _calcDelta = function(touches) {
        var average;
        average = _average(touches);
        return {
          x: average.x - _start.x,
          y: average.y - _start.y
        };
      };
      _average = function(touches) {
        var touch, x, y, _i, _len;
        x = 0;
        y = 0;
        for (_i = 0, _len = touches.length; _i < _len; _i++) {
          touch = touches[_i];
          x += parseInt(touch.x);
          y += parseInt(touch.y);
        }
        return {
          x: x / touches.length,
          y: y / touches.length
        };
      };
      _check = function(moving) {
        if (moving) {
          return base.trigger(_target, "dragging", _last);
        } else if (Math.abs(_last.delta.x) > GAP || Math.abs(_last.delta.y) > GAP) {
          return base.trigger(_target, "drag", _last);
        }
      };
      return {
        start: start,
        move: move,
        end: end
      };
    })(Quo.Gestures)
  });

}).call(this);


/*
Quo Pinch Gestures: pinch, pinching, pinchIn, pinchOut

@namespace Quo.Gestures
@class Pinch

@author Javier Jimenez Villar <javi@tapquo.com> || @soyjavi
 */

(function() {
  "use strict";
  Quo.Gestures.add({
    name: "pinch",
    events: ["pinch", "pinching", "pinchIn", "pinchOut"],
    handler: (function(base) {
      var GAP, end, move, start, _check, _distance, _last, _start, _target;
      GAP = 20;
      _target = null;
      _start = null;
      _last = null;
      start = function(target, data) {
        if (data.length === 2) {
          _target = target;
          return _start = _distance(data[0], data[1]);
        }
      };
      move = function(target, data) {
        var distance;
        if (_start && data.length === 2) {
          distance = _distance(data[0], data[1]);
          _last = {
            touches: data,
            delta: distance - _start
          };
          return _check(true);
        }
      };
      end = function(target, data) {
        if (_start && _last) {
          _check(false);
          _start = null;
          return _last = null;
        }
      };
      _distance = function(A, B) {
        return Math.sqrt((B.x - A.x) * (B.x - A.x) + (B.y - A.y) * (B.y - A.y));
      };
      _check = function(moving) {
        var ev;
        if (moving) {
          return base.trigger(_target, "pinching", _last);
        } else if (Math.abs(_last.delta) > GAP) {
          base.trigger(_target, "pinch", _last);
          ev = _last.delta > 0 ? "pinchOut" : "pinchIn";
          return base.trigger(_target, ev, _last);
        }
      };
      return {
        start: start,
        move: move,
        end: end
      };
    })(Quo.Gestures)
  });

}).call(this);


/*
Quo Rotation Gestures: rotate, rotating, rotateLeft, rotateRight

@namespace Quo.Gestures
@class Rotation

@author Javier Jimenez Villar <javi@tapquo.com> || @soyjavi
 */

(function() {
  "use strict";
  Quo.Gestures.add({
    name: "rotation",
    events: ["rotate", "rotating", "rotateLeft", "rotateRight"],
    handler: (function(base) {
      var GAP, ROTATION_LIMIT, end, move, start, _check, _last, _num_rotations, _rotation, _sign, _start, _target;
      GAP = 5;
      ROTATION_LIMIT = 20;
      _target = null;
      _num_rotations = 0;
      _start = null;
      _last = null;
      start = function(target, data) {
        if (data.length === 2) {
          _target = target;
          _num_rotations = 0;
          return _start = _rotation(data[0], data[1]);
        }
      };
      move = function(target, data) {
        var delta;
        if (_start && data.length === 2) {
          delta = _rotation(data[0], data[1]) - _start;
          if (_last && Math.abs(_last.delta - delta) > ROTATION_LIMIT) {
            delta += 360 * _sign(_last.delta);
          }
          if (Math.abs(delta) > 360) {
            _num_rotations++;
            delta -= 360 * _sign(_last.delta);
          }
          _last = {
            touches: data,
            delta: delta,
            rotationsCount: _num_rotations
          };
          return _check(true);
        }
      };
      end = function(target, data) {
        if (_start && _last) {
          _check(false);
          _target = null;
          _num_rotations = 0;
          _start = null;
          _last = null;
          return _start = null;
        }
      };
      _sign = function(num) {
        if (num < 0) {
          return -1;
        } else {
          return 1;
        }
      };
      _rotation = function(A, B) {
        var theta;
        theta = Math.atan2(A.y - B.y, A.x - B.x);
        return (theta < 0 ? theta + 2 * Math.PI : theta) * 180 / Math.PI;
      };
      _check = function(moving) {
        var ev;
        if (moving) {
          return base.trigger(_target, "rotating", _last);
        } else if (Math.abs(_last.delta) > GAP) {
          base.trigger(_target, "rotate", _last);
          ev = _last.delta > 0 ? "rotateRight" : "rotateLeft";
          return base.trigger(_target, ev, _last);
        }
      };
      return {
        start: start,
        move: move,
        end: end
      };
    })(Quo.Gestures)
  });

}).call(this);


/*
Quo Swipe Gestures: swipe, swiping, swipeLeft, swipeRight, swipeUp, swipeDown
New gestures added: swipingHorizontal, swipingVertical

@namespace Quo.Gestures
@class Swipe

@author Javier Jimenez Villar <javi@tapquo.com> || @soyjavi
 */

(function() {
  "use strict";
  Quo.Gestures.add({
    name: "swipe",
    events: ["swipe", "swipeLeft", "swipeRight", "swipeUp", "swipeDown", "swiping", "swipingHorizontal", "swipingVertical"],
    handler: (function(base) {
      var GAP, end, move, start, _check, _getInitialAxis, _last, _start, _start_axis, _target;
      GAP = 20;
      _target = null;
      _start = null;
      _start_axis = null;
      _last = null;
      start = function(target, data) {
        if (data.length === 1) {
          _target = target;
          _start = data[0];
          return _last = null;
        }
      };
      move = function(target, data) {
        var delta, is_first;
        if (data.length === 1) {
          console.log(target);
          delta = {
            x: data[0].x - _start.x,
            y: data[0].y - _start.y
          };
          is_first = _last === null;
          _last = {
            x: data[0].x,
            y: data[0].y,
            delta: delta
          };
          return _check(true, is_first);
        } else {
          return _last = null;
        }
      };
      end = function(target, data) {
        if (_last) {
          _check(false);
          return _last = null;
        }
      };
      _check = function(moving, first_move) {
        var direction, directions, _i, _len, _results;
        if (first_move == null) {
          first_move = false;
        }
        if (moving) {
          if (first_move) {
            _start_axis = _getInitialAxis(_last.delta.x, _last.delta.y);
          }
          if (_start_axis !== null) {
            base.trigger(_target, "swiping" + _start_axis, _last);
          }
          return base.trigger(_target, "swiping", _last);
        } else {
          directions = [];
          if (Math.abs(_last.delta.y) > GAP) {
            directions.push(_last.delta.y < 0 ? "Up" : "Down");
          }
          if (Math.abs(_last.delta.x) > GAP) {
            directions.push(_last.delta.x < 0 ? "Left" : "Right");
          }
          if (directions.length) {
            base.trigger(_target, "swipe", _last);
            _results = [];
            for (_i = 0, _len = directions.length; _i < _len; _i++) {
              direction = directions[_i];
              _results.push(base.trigger(_target, "swipe" + direction, _last));
            }
            return _results;
          }
        }
      };
      _getInitialAxis = function(x, y) {
        var axis;
        axis = null;
        if (Math.round(Math.abs(x / y)) >= 2) {
          axis = "Horizontal";
        } else if (Math.round(Math.abs(y / x)) >= 2) {
          axis = "Vertical";
        }
        return axis;
      };
      return {
        start: start,
        move: move,
        end: end
      };
    })(Quo.Gestures)
  });

}).call(this);
