
/*
Basic Quo Module

@namespace Quo
@class Base

@author Javier Jimenez Villar <javi@tapquo.com> || @soyjavi
 */

(function() {
  "use strict";
  var Quo;

  Quo = (function() {
    var $$, CLASS_SELECTOR, ELEMENT_TYPES, EMPTY_ARRAY, HTML_CONTAINERS, ID_SELECTOR, IS_HTML_FRAGMENT, OBJECT_PROTOTYPE, TABLE, TABLE_ROW, TAG_SELECTOR, _compact, _flatten, _fragment, _getDOMObject, _instance, _isOwnProperty;
    EMPTY_ARRAY = [];
    OBJECT_PROTOTYPE = Object.prototype;
    IS_HTML_FRAGMENT = /^\s*<(\w+|!)[^>]*>/;
    ELEMENT_TYPES = [1, 9, 11];
    CLASS_SELECTOR = /^\.([\w-]+)$/;
    ID_SELECTOR = /^#[\w\d-]+$/;
    TAG_SELECTOR = /^[\w-]+$/;
    TABLE = document.createElement('table');
    TABLE_ROW = document.createElement('tr');
    HTML_CONTAINERS = {
      "tr": document.createElement("tbody"),
      "tbody": TABLE,
      "thead": TABLE,
      "tfoot": TABLE,
      "td": TABLE_ROW,
      "th": TABLE_ROW,
      "*": document.createElement("div")
    };

    /*
    Basic Instance of QuoJS
    @method $$
    @param  {string/instance} [OPTIONAL] Selector for handler
    @param  {string} [OPTIONAL] Children in selector
     */
    $$ = function(selector, children) {
      var dom;
      if (!selector) {
        return _instance();
      } else if ($$.toType(selector) === "function") {
        return $$(document).ready(selector);
      } else {
        dom = _getDOMObject(selector, children);
        return _instance(dom, selector);
      }
    };

    /*
    Basic Instance of QuoJS
    @method query
    @param  {string/instance} [OPTIONAL] Selector for handler
    @param  {string} [OPTIONAL] Children in selector
     */
    $$.query = function(domain, selector) {
      var elements;
      if (CLASS_SELECTOR.test(selector)) {
        elements = domain.getElementsByClassName(selector.replace(".", ""));
      } else if (TAG_SELECTOR.test(selector)) {
        elements = domain.getElementsByTagName(selector);
      } else if (ID_SELECTOR.test(selector) && domain === document) {
        elements = domain.getElementById(selector.replace("#", ""));
        if (!elements) {
          elements = [];
        }
      } else {
        elements = domain.querySelectorAll(selector);
      }
      if (elements.nodeType) {
        return [elements];
      } else {
        return Array.prototype.slice.call(elements);
      }
    };
    $$.extend = function(target) {
      Array.prototype.slice.call(arguments, 1).forEach(function(source) {
        var key, _results;
        _results = [];
        for (key in source) {
          _results.push(target[key] = source[key]);
        }
        return _results;
      });
      return target;
    };
    $$.toType = function(obj) {
      return OBJECT_PROTOTYPE.toString.call(obj).match(/\s([a-z|A-Z]+)/)[1].toLowerCase();
    };
    $$.each = function(elements, callback) {
      var element, i, key, _i, _len;
      i = void 0;
      key = void 0;
      if ($$.toType(elements) === "array") {
        for (i = _i = 0, _len = elements.length; _i < _len; i = ++_i) {
          element = elements[i];
          if (callback.call(element, i, element) === false) {
            elements;
          }
        }
      } else {
        for (key in elements) {
          if (callback.call(elements[key], key, elements[key]) === false) {
            elements;
          }
        }
      }
      return elements;
    };
    $$.map = function(elements, callback) {
      var i, key, value, values;
      values = [];
      i = void 0;
      key = void 0;
      if ($$.toType(elements) === "array") {
        i = 0;
        while (i < elements.length) {
          value = callback(elements[i], i);
          if (value != null) {
            values.push(value);
          }
          i++;
        }
      } else {
        for (key in elements) {
          value = callback(elements[key], key);
          if (value != null) {
            values.push(value);
          }
        }
      }
      return _flatten(values);
    };
    $$.mix = function() {
      var arg, argument, child, len, prop;
      child = {};
      arg = 0;
      len = arguments.length;
      while (arg < len) {
        argument = arguments[arg];
        for (prop in argument) {
          if (_isOwnProperty(argument, prop) && argument[prop] !== undefined) {
            child[prop] = argument[prop];
          }
        }
        arg++;
      }
      return child;
    };
    _instance = function(dom, selector) {
      if (selector == null) {
        selector = "";
      }
      dom = dom || EMPTY_ARRAY;
      dom.selector = selector;
      dom.__proto__ = _instance.prototype;
      return dom;
    };
    _getDOMObject = function(selector, children) {
      var domain, type;
      domain = null;
      type = $$.toType(selector);
      if (type === "array") {
        domain = _compact(selector);
      } else if (type === "string" && IS_HTML_FRAGMENT.test(selector)) {
        domain = _fragment(selector.trim(), RegExp.$1);
        selector = null;
      } else if (type === "string") {
        domain = $$.query(document, selector);
        if (children) {
          if (domain.length === 1) {
            domain = $$.query(domain[0], children);
          } else {
            domain = $$.map(function() {
              return $$.query(domain, children);
            });
          }
        }
      } else if (ELEMENT_TYPES.indexOf(selector.nodeType) >= 0 || selector === window) {
        domain = [selector];
        selector = null;
      }
      return domain;
    };
    _fragment = function(markup, tag) {
      var container;
      if (tag == null) {
        tag = "*";
      }
      if (!(tag in HTML_CONTAINERS)) {
        tag = "*";
      }
      container = HTML_CONTAINERS[tag];
      container.innerHTML = "" + markup;
      return $$.each(Array.prototype.slice.call(container.childNodes), function() {
        return container.removeChild(this);
      });
    };
    _compact = function(items) {
      return items.filter(function(item) {
        if (item != null) {
          return item;
        }
      });
    };
    _flatten = function(array) {
      if (array.length > 0) {
        return EMPTY_ARRAY.concat.apply(EMPTY_ARRAY, array);
      } else {
        return array;
      }
    };
    _isOwnProperty = function(object, property) {
      return OBJECT_PROTOTYPE.hasOwnProperty.call(object, property);
    };
    _instance.prototype = $$.fn = {};
    $$.fn.each = function(callback) {
      this.forEach(function(element, index) {
        return callback.call(element, index, element);
      });
      return this;
    };
    $$.fn.filter = function(selector) {
      return $$(EMPTY_ARRAY.filter.call(this, function(el) {
        return el.parentNode && $$.query(el.parentNode, selector).indexOf(el) >= 0;
      }));
    };
    $$.fn.forEach = EMPTY_ARRAY.forEach;
    $$.fn.indexOf = EMPTY_ARRAY.indexOf;
    return $$;
  })();

  this.Quo = this.$$ = Quo;

}).call(this);
