(function() {
  describe("Core", function() {
    var body;
    body = void 0;
    beforeEach(function() {
      body = $$(document.body);
      return document.body.innerHTML = "<section id=\"quo\" class=\"quo\"></section>";
    });
    it("is healthy", function() {
      expect(Quo).toBeTruthy();
      return expect($$).toBeTruthy();
    });
    it("select body", function() {
      expect(body).toBeTruthy();
      return expect(body.length > 0).toBeTruthy();
    });
    it("select by tag element", function() {
      var tag;
      tag = $$("section");
      expect(tag).toBeTruthy();
      return expect(tag.length > 0).toBeTruthy();
    });
    it("select by Id", function() {
      var id;
      id = $$("#quo");
      expect(id).toBeTruthy();
      return expect(id.length > 0).toBeTruthy();
    });
    it("select by className", function() {
      var className;
      className = $$(".quo");
      expect(className).toBeTruthy();
      return expect(className.length > 0).toBeTruthy();
    });
    return it("if select doesn't exists array is empty.", function() {
      var el;
      el = $$("article");
      return expect(el.length === 0).toBeTruthy();
    });
  });

  describe("Ajax", function() {
    var url;
    url = "http://tapquo.com";
    it("can configure ajax settings", function() {
      $$.ajaxSettings = {
        async: false,
        timeout: 1000
      };
      expect($$.ajaxSettings.async).toBeFalsy();
      return expect($$.ajaxSettings.timeout).toEqual(1000);
    });
    it("can make a ajax request", function() {
      spyOn($$, "ajax");
      $$.ajax({
        url: url
      });
      return expect($$.ajax.mostRecentCall.args[0]["url"]).toEqual(url);
    });
    it("should execute the success callback function", function() {
      var _error, _success;
      spyOn($$, "ajax").andCallFake(function(options) {
        return options.success();
      });
      _success = jasmine.createSpy();
      _error = jasmine.createSpy();
      $$.ajax({
        url: "http:/asdad",
        success: _success,
        error: _error
      });
      expect(_success).toHaveBeenCalled();
      return expect(_error).not.toHaveBeenCalled();
    });
    it("should execute the error callback function", function() {
      var _error;
      spyOn($$, "ajax").andCallFake(function(options) {
        return options.error();
      });
      _error = jasmine.createSpy();
      $$.ajax({
        url: url,
        error: _error
      });
      return expect(_error).toHaveBeenCalled();
    });
    return it("can serialize parameters", function() {
      var parameters, serializedParameters;
      parameters = {
        format: 'json',
        key: '1980',
        page: 0,
        ordered: true
      };
      serializedParameters = "format=json&key=1980&page=0&ordered=true";
      return expect($$.serialize(parameters)).toEqual(serializedParameters);
    });
  });

  describe("CSS", function() {
    var el;
    el = void 0;
    beforeEach(function() {
      document.body.innerHTML = "<section class=\"quo\"></section>";
      return el = $$("section");
    });
    it("add class to a given element", function() {
      el.addClass("new");
      return expect(el.hasClass("new")).toBeTruthy();
    });
    it("remove class to a given element", function() {
      el.removeClass("quo");
      return expect(el.hasClass("quo")).not.toBeTruthy();
    });
    it("toggle class to a given element", function() {
      expect(el.hasClass("quo")).toBeTruthy();
      el.toggleClass("quo");
      return expect(el.hasClass("quo")).not.toBeTruthy();
    });
    it("test if class exists in a given element", function() {
      return expect(el.hasClass("quo")).toBeTruthy();
    });
    it("returns a list of class styles", function() {
      var classList;
      classList = {
        '0': 'quo',
        length: 1
      };
      return expect(el.listClass()["0"]).toEqual(classList["0"]);
    });
    it("set a stylesheet property in a given element", function() {
      var color, font;
      color = "blue";
      el.style("background-color", color);
      expect(el.style("background-color")).toEqual(color);
      font = "10px";
      el.css("font-size", font);
      return expect(el.style("font-size")).toEqual(font);
    });
    return it("set a stylesheet vendor-prefix property in a given element", function() {
      var value;
      value = "black 1em 1em";
      el.vendor("box-shadow", value);
      return expect(el.style("-webkit-box-shadow")).toEqual(value);
    });
  });

  describe("Element", function() {
    var el;
    el = void 0;
    beforeEach(function() {
      document.body.innerHTML = "<input type=\"text\" class=\"quo\" data-quo=\"js\" value=\"1980\"/>";
      return el = $$("input");
    });
    it("can get a attribute of a given element", function() {
      var attribute;
      attribute = "class";
      return expect(el.attr(attribute)).toEqual("quo");
    });
    it("can set a attribute in a given element", function() {
      var attribute, value;
      attribute = "data-framework";
      value = "quojs";
      el.attr(attribute, value);
      return expect(el.attr(attribute)).toEqual(value);
    });
    it("can remove a attribute in a given element", function() {
      var attribute;
      attribute = "class";
      el.removeAttr(attribute);
      return expect(el.attr(attribute)).toEqual(null);
    });
    it("can get a data attribute in a given element", function() {
      var attribute;
      attribute = "quo";
      return expect(el.data(attribute)).toEqual("js");
    });
    it("can set a data attribute in a given element", function() {
      var attribute, value;
      attribute = "framework";
      value = "quojs";
      el.data(attribute, value);
      return expect(el.data(attribute)).toEqual(value);
    });
    it("can remove a data attribute in a given element", function() {
      var attribute;
      attribute = "quo";
      el.removeData(attribute);
      return expect(el.attr(attribute)).toEqual(null);
    });
    it("get a value of a given element", function() {
      var value;
      value = "1980";
      return expect(el.val()).toEqual(value);
    });
    it("set a value of a given element", function() {
      var value;
      value = "1983";
      el.val(value);
      return expect(el.val()).toEqual(value);
    });
    it("can show a given element", function() {
      el.show();
      return expect(el.style("display")).toEqual("block");
    });
    it("can hide a given element", function() {
      el.hide();
      return expect(el.style("display")).toEqual("none");
    });
    return it("get a offset of a given element", function() {
      var offset;
      offset = el.offset();
      expect(offset.left != null).toBeTruthy();
      expect(offset.top != null).toBeTruthy();
      expect(offset.width != null).toBeTruthy();
      return expect(offset.height != null).toBeTruthy();
    });
  });

  describe("Environment", function() {
    it("know if device it's mobile or not", function() {
      return expect($$.isMobile()).not.toBeTruthy();
    });
    return it("get environment attributes", function() {
      var environment;
      environment = $$.environment();
      expect(environment.browser != null).toBeTruthy();
      expect(environment.os != null).not.toEqual(null);
      expect(environment.screen != null).toBeTruthy();
      return expect(environment.isMobile != null).toBeTruthy();
    });
  });

  describe("Output", function() {
    var el, markup;
    el = void 0;
    markup = "<header>\n  <nav>\n    <a href=\"#\">One</a>\n    <a href=\"#\">Two</a>\n    <a href=\"#\">Three</a>\n  </nav>\n</header>\n<article></article>\n<article></article>\n<footer></footer>";
    beforeEach(function() {
      document.body.innerHTML = "<section class=\"quo\">" + markup + "</section>";
      return el = $$("section");
    });
    it("can set/get text to a given element", function() {
      var value;
      value = "hello";
      el.text(value);
      return expect(el.text()).toEqual(value);
    });
    it("can get html value to a given element", function() {
      return expect(el.html()).toEqual(markup);
    });
    it("can set html value to a given element", function() {
      var new_markup;
      new_markup = "<aside></aside>";
      el.html(new_markup);
      return expect(el.html()).toEqual(new_markup);
    });
    it("can remove the set of matched elements to a given element", function() {
      el.remove();
      return expect($$("section").length).toEqual(0);
    });
    it("can Remove all child nodes of the set of matched elements to a given element", function() {
      el.empty();
      return expect(el.html()).toEqual("");
    });
    it("can append a html element to a given element", function() {
      var li;
      li = ["<li>1</li>", "<li>2</li>"];
      el.html(li[0]);
      el.append(li[1]);
      return expect(el.html()).toEqual(li[0] + li[1]);
    });
    it("can prepend a html element to a given element", function() {
      var li;
      li = ["<li>1</li>", "<li>2</li>"];
      el.html(li[0]);
      el.prepend(li[1]);
      return expect(el.html()).toEqual(li[1] + li[0]);
    });
    return it("can replace each element in a given element", function() {
      var new_markup;
      new_markup = "<header>\n  <nav>\n    <a href=\"#\">One</a>\n    <a href=\"#\">Two</a>\n    <a href=\"#\">Three</a>\n  </nav>\n</header>\n<article></article>\n<article></article>\n<aside></aside>";
      $$("section > footer").replaceWith("<aside></aside>");
      return expect(el.html()).toEqual(new_markup);
    });
  });

  describe("Output", function() {
    var el;
    el = void 0;
    beforeEach(function() {
      document.body.innerHTML = "<section class=\"quo\">\n  <header></header>\n</section>";
      return el = $$("section");
    });
    it("can get the descendants of each element in the current instance", function() {
      var children;
      children = el.find("header");
      return expect(children.length > 0).toBeTruthy();
    });
    it("can get the parent of each element in the current instance", function() {
      var children;
      children = el.find("header");
      return expect(children.parent().html()).toEqual(el.html());
    });
    it("can get the children of each element in the current instance", function() {
      var children, header;
      children = el.children();
      header = $$("section > header");
      expect(children.html()).toEqual(header.html());
      el.append("<footer></footer>");
      children = el.children("header");
      return expect(children.html()).toEqual(header.html());
    });
    it("can retrieve the DOM elements matched by the QuoJS object.", function() {
      var dom;
      dom = el.get(0);
      expect(dom).toBeTruthy();
      dom = el.get(1);
      return expect(dom).not.toBeTruthy();
    });
    it("can reduce the set of matched elements to the first in the set.", function() {
      el.append("<footer></footer>");
      expect(el.children().length).toEqual(2);
      return expect(el.children().first().length).toEqual(1);
    });
    it("can reduce the set of matched elements to the final one in the set.", function() {
      var last;
      el.append("<footer>quojs</footer>");
      last = el.children().last();
      expect(last.length).toEqual(1);
      return expect(last.text()).toEqual("quojs");
    });
    it("can get the first element that matches the selector by testing the element itself and traversing up through its ancestors", function() {
      var dom, header;
      header = $$("section header");
      dom = header.closest("section");
      return expect(dom.html()).toEqual(el.html());
    });
    it("can get the immediately following sibling of each element in the instance", function() {
      var footer, header;
      el.append("<footer></footer>");
      header = $$("section header");
      footer = $$("section footer");
      return expect(header.next().html()).toEqual(footer.html());
    });
    return it("can get the immediately preceding sibling of each element in the instance", function() {
      var footer, header;
      el.append("<footer></footer>");
      header = $$("section header");
      footer = $$("section footer");
      return expect(footer.prev().html()).toEqual(header.html());
    });
  });

}).call(this);
